#include "MemoryPool.h"

namespace AMDiS {

  ::std::vector<MemoryPool*> MemoryPool::containerArray;
  unsigned int MemoryPool::increment = 8*1024-12;

  void MemoryPool::grow()
  {
    int chunkSize = elementSize * localIncrement;

    char *start = new char[chunkSize];
    char *last  = &start[(localIncrement - 1) * elementSize];

    char *p;

    for(p=start; p < last; p+=elementSize) 
      {
	reinterpret_cast<Link*>( p)->next = reinterpret_cast<Link*>(p + elementSize);
      }
    reinterpret_cast<Link*>( last)->next = NULL;
    head = reinterpret_cast<Link*>( start);

    // create new block info and store it in the list
    MemoryBlock *newBlock = new MemoryBlock;
    newBlock->address = start;
    newBlock->next    = firstMemoryBlock;
    firstMemoryBlock  = newBlock;
  }

  void MemoryPool::compress()
  {
    MemoryBlock *block;

    for(block = firstMemoryBlock; block != NULL; block = block->next) 
      block->usedCount = localIncrement;

    // count used memory in each block
    Link *nextFree;
    for(nextFree = head; nextFree != NULL; nextFree = nextFree->next) {
      // get memory block of nextFree
      for(block = firstMemoryBlock; block != NULL; block = block->next) {
	if((reinterpret_cast<char*>(nextFree) >= block->address) && 
	   (reinterpret_cast<char*>( nextFree) < block->address+localIncrement*elementSize)) {
	  // block found
	  break;
	}
      }
      if(block == NULL) {
	ERROR_EXIT("no memory block found for nextFree\n");
      } else {
	block->usedCount--;
	if(block->usedCount < 0) {    
	  ERROR_EXIT("usedCount < 0 !\n");
	}
      }
    }
  
    // delete unused blocks
    MemoryBlock *lastBlock = NULL;
    for(block=firstMemoryBlock; block != NULL; block=block->next) {
      if(block->usedCount == 0) {
	if(lastBlock) {
	  lastBlock->next = block->next;
	} else {
	  firstMemoryBlock = block->next;
	}
	delete[] block->address;
	delete block;
      } else {
	lastBlock = block;
      }
    }
  }

}
