// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ValueWriter.h */

#ifndef AMDIS_VALUEWRITER_H
#define AMDIS_VALUEWRITER_H

#include <list>

#include "Global.h"
#include "FixVec.h"
#include "Flag.h"
#include "Mesh.h"
#include "DataCollector.h"

namespace AMDiS {

  class DOFAdmin;
  class ElInfo;

  template<typename T> class DOFVector;

  // ============================================================================
  // ===== class ValueWriter ====================================================
  // ============================================================================

  /** \ingroup Output
   * \brief
   * ValueWriter is a static class which writes the values of a DOFVector
   * values to an ascii file named values->name.'dat'. This output is done
   * via two leaf-traversals of values->feSpace->mesh. In the first traversal
   * the values at the vertices are printed, in the second these at the
   * interpolation points of each element. For a closer disription of the
   * output format see (...link fehlt noch)
   */
  class ValueWriter
  {
  public:
    /** \brief
     * Writes DOFVector values to values->feSpace->mesh.
     */
    static void writeValues(DataCollector *dc,
			    const ::std::string filename,
			    double time = 0.0,
			    int level = -1,
			    Flag traverseFlag = Mesh::CALL_LEAF_EL,
			    bool (*writeElem)(ElInfo*) = NULL);
			   
  protected:
    /** \brief
     * File to which the values should be written
     */
    static FILE *valueFile;

    /** \brief
     * Global interpolation point indexing
     */
    static DOFVector<int> *interpPointInd;

    /** \brief
     * list of coords for each dof
     */
    static DOFVector< ::std::list<WorldVector<double> > > *dofCoords;

    /** \brief
     * DOFAdmin of values
     */
    static const DOFAdmin *admin;

    /** \brief
     * Pointer to have a global access to values
     */
    static DOFVector<double> *valueVec;

    /** \brief
     * Total number of interpolation points.
     */
    static int ni;
  };

}

#endif // AMDIS_VALUEWRITER_H
